/* Tous les fichiers présents dans ce répertoire sont soumis au copyright suivant :
(c) Copyright : Samuel Bizien, 2008
Ils sont disponibles sous licence GNU/GPL version 3 (ou suivante), dont vous pouvez
obtenir une copie à l'adresse : http://www.gnu.org/licenses/gpl.txt */

#include <stdio.h>
#include "parametres.h"
#include "beryl.h"

#define XMIN -1.22
#define XMAX 0.11
#define YMIN -0.7
#define YMAX 0.7
#define U0_RE 1.0
#define U0_IM 0.0

/*
  k    : ( xmin, xmax,  ymin, ymax) 
    1  : ( -1.5,  0.4,  -1.4,  1.4)
  0.999: ( -1.7,  0.6, -1.37, 1.37)
  0.995: (-1.62, 0.53, -1.36, 1.36)
   0.99: (-1.95, 1.21,  -1.4,  1.4)
+ 0.985: (-1.22, 0.11, -0.7, 0.7)
   0.98: (-1.22, 0.12,-0.705,0.705)
   0.97: (-1.125,0.13,-0.725,0.725)
   0.96: (-1.28, 0.14, -0.74, 0.74)
   0.95: (-1.91, 1.29,  -1.4,  1.4)
   0.94: (-1.35, 0.15, -0.78, 0.78)
   0.93: (-1.36, 0.17,   -0.8, 0.8)
+  0.92: (-1.37, 0.18, -0.82, 0.82)
+  0.91: (-1.39, 0.19, -0.89, 0.89)
   0.9 : (-1.86, 0.56, -1.88, 1.88)
-  0.89: (-1.48, 0.21,  -0.9,  0.9)
-  0.88: (-1.48, 0.22,-0.925,0.925)
   0.87: (-1.48, 0.23, -0.94, 0.94)
-  0.86: (-1.49, 0.25, -0.95, 0.95)
   0.85: (-1.78, 0.53, -1.93, 1.93)
-  0.84: (-1.53, 0.27,-0.975,0.975)
   0.83: (-1.55, 0.28, -0.99, 0.99)
   0.82: (-1.58,0.295,  -1.0,  1.0)
   0.81: (-2.71, 1.43, -2.03, 2.03)
   0.8 : (-2.79, 1.33, -2.06, 2.06)
   0.79: (-2.8,  1.34, -2.08, 2.08)
   0.75: (-2.85,  1.3,  -2.2,  2.2)
   0.7 : ( -3.5,  1.5,  -2.5,  2.5)
   0.65: (-2.46,  0.6,  -1.3,  1.3)
   0.6 : (-3.56, 1.75, -2.85, 2.85)
   0.55: (-2.63, 0.87, -1.58, 1.58)
   0.5 : ( -2.9,  1.1,    -2,    2)
*/


long int compter_boites(double pas, double xmin, double xmax, double ymin, double ymax, int maxiter) {
  register unsigned int i, j;
  register short int etat_pt_prec, etat_pt, somme_etats;
  unsigned int jmax = (((int) (ymax-ymin)  / pas) == ((ymax-ymin)  / pas)) ? (ymax - ymin)/pas : (int) ((ymax - ymin)/pas) + 1;
  unsigned int imax = (((int) (xmax-xmin)  / pas) == ((xmax-xmin)  / pas)) ? (xmax - xmin)/pas : (int) ((xmax - xmin)/pas) + 1;
  register double xstep = pas, ystep = pas;
  register unsigned long int nombre_boites = 0;
  short int ligne[imax+1];
  register double x=xmin, y=ymin;
  for(i=0; i<=imax; i++) {
    ligne[i] = est_dans_beryl(U0_RE,U0_IM,x, y, maxiter);
    x += xstep;
  }
  for(j=1; j<=jmax; j++) {
    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bLigne : %i/%i", j, jmax);
    y += ystep;
    x = xmin;
    etat_pt_prec = est_dans_beryl(U0_RE, U0_IM, x, y, maxiter); 
    for(i=1; i<=imax; i++) {
      x += xstep;
      etat_pt = est_dans_beryl(U0_RE,U0_IM,x, y, maxiter); 
      somme_etats = etat_pt_prec + etat_pt + ligne[i-1] + ligne[i];
      if(somme_etats != 0 && somme_etats != 4) {
	nombre_boites += 1;
	    }
      ligne[i-1] = etat_pt_prec;
      etat_pt_prec = etat_pt;
    }
    ligne[i-1] = etat_pt;
    }
  printf("\nFenetre :  (%f, %f) (%f, %f)\n", xmin, ymin, xmax, ymax);
  printf("Boites : %li, precision : %f\n", nombre_boites, pas);
  printf(" --- \n");
  return nombre_boites;
}

int main () {
  double precision;
  short int i, j;
  FILE *fichier;
  unsigned long int nb_boites;
  for (precision=MIN_PRECISION; precision > MAX_PRECISION; precision /= 2) {
    for(j=0; j< NB_MESURES; j++) {
      for (i=0; i< NB_MESURES; i++) {
	nb_boites = compter_boites(precision, XMIN - i*precision/NB_MESURES, XMAX, YMIN - j*precision/NB_MESURES, YMAX, 200 + 100/precision); /* 20 + 30/pr */
	fichier = fopen("data-0.985-1-0", "a"); 
	fprintf(fichier, "%f; %li;\n", 1/precision, nb_boites);
	fclose(fichier);
      }
    }
  }
  return 0;
}
